const BusinessSettingModel = require("../models/BusinessSettingModel");
const BusinessSettingResource = require("../utils/resources/BusinessSettingResource");
const ApiError = require("../utils/apiError");

exports.GetConfig = async (req, res, next) => {
  const settings = await BusinessSettingModel.find();
  if (!settings) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }

  const data = await BusinessSettingResource.build(settings);
  res.status(200).json(data);
};

exports.Add = async (req, res) => {
  try {
    const { key, value } = req.body;

    await BusinessSettingModel.create({
      key,
      value,
    });
    res.status(200).json({
      success: true,
      message: "created successfully",
    });
  } catch (error) {
    console.error("Error adding setting:", error);
    res.status(500).json({ error: error.message });
  }
};

exports.Update = async (req, res, next) => {
  try {
    const payload = req.body;

    if (!payload || typeof payload !== "object") {
      return next(
        new ApiError("Invalid data format. Expected key-value pairs.", 400)
      );
    }

    // Prepare bulk update operations
    const bulkOps = Object.entries(payload).map(([key, value]) => ({
      updateOne: {
        filter: { key },
        update: { $set: { value } },
        upsert: false, // Set to true if you want to create if not exists
      },
    }));

    // Execute bulkWrite
    await BusinessSettingModel.bulkWrite(bulkOps);

    // Fetch all updated settings
    const updatedKeys = Object.keys(payload);
    const updatedSettings = await BusinessSettingModel.find();

    // Format using your resource
    const data = await BusinessSettingResource.build(updatedSettings);

    res.status(200).json(data);
  } catch (error) {
    next(new ApiError(error.message, 500));
  }
};
