const Message = require("../models/MessageModel");
const Conversation = require("../models/ConversationModel");
const { send_notification } = require("./NotificationController");
const {
  check_conversation,
  send_message,
  GetMessage,
  getHomeConversations,
} = require("../utils/logic/chat");

exports.sendMessage = async (req, res) => {
  try {
    const { receiver, content, service, offer } = req.body;
    const sender = req.user.id;

    const io = req.app.get("socketio");
    const chatNamespace = io.of("/SocketChat");

    const conversation = await check_conversation(
      sender,
      receiver,
      service,
      offer
    );
    const conversation_id = conversation._id;
    const chatRoom = `chat_${conversation_id}`;

    const userSockets = req.app.get("userSockets");
    const senderSocketId = userSockets.get(sender);

    console.log(
      "All Connected Sockets:",
      Array.from(chatNamespace.sockets.keys())
    );

    if (!senderSocketId) {
      console.log(`⚠️ Sender ${sender} is not connected to any socket.`);
      return res.status(400).json({ error: "Sender is not connected." });
    }

    const senderSocket = chatNamespace.sockets.get(senderSocketId);

    if (typeof content !== "undefined") {
      if (!conversation || conversation.status === false) {
        return res.status(400).json({ error: "Conversation is closed." });
      }
      await send_message(sender, conversation_id, content);
    }
    /// console.log('senderSocket',senderSocket);
    console.log("senderSocketId", senderSocketId);
    senderSocket.join(chatRoom);
    console.log(`🔵 Sender ${sender} joined ${chatRoom}`);
    const room = io.sockets.adapter.rooms.get(chatRoom);

    if (room) {
      if (room.size == 2) {
        const receiverSocketId = userSockets.get(receiver);
        if (receiverSocketId) {
          const receiverSocket = io.sockets.sockets.get(receiverSocketId);
          if (receiverSocket) {
            const chatDataForReceiver = await GetMessage(
              conversation_id,
              receiver
            );
            receiverSocket.emit(
              "Message",
              JSON.stringify({ Message: chatDataForReceiver })
            );
          }
        }
      } else if (room.size == 1 && content) {
        await send_notification(sender, receiver, content, service, "message");
      }
    }
    const chatData = await GetMessage(conversation_id, sender);
    senderSocket.emit("Message", JSON.stringify({ Message: chatData }));

    // const home = await getHomeConversations(receiver);
    // io.to(`chat_user_${receiver}`).emit("Home", JSON.stringify({ home }));

    res.status(201).json({ message: "Message sent" });
  } catch (error) {
    console.error("Error sending message:", error);
    res.status(500).json({ error: "Server error" });
  }
};

exports.home = async (req, res) => {
  try {
    const sender = req.user.id;
    console.log(sender);

    const io = req.app.get("socketio");

    const home = await getHomeConversations(sender);
    io.to(`user_${sender}`).emit("Home", JSON.stringify({ home }));

    res.status(201).json({ message: "home" });
  } catch (error) {
    console.error("Error sending message:", error);
    res.status(500).json({ error: "Server error" });
  }
};

exports.leave = async (req, res) => {
  try {
    const sender = req.user.id;
    const chatRoom = `chat_${req.body.conversation_id}`;
    const io = req.app.get("socketio");

    const userSockets = req.app.get("userSockets");
    const senderSocketId = userSockets.get(sender);

    if (!senderSocketId) {
      console.log(`⚠️ Sender ${sender} is not connected to any socket.`);
      return res.status(400).json({ error: "Sender is not connected." });
    }

    const senderSocket = io.sockets.sockets.get(senderSocketId);
    if (!senderSocket) {
      console.log(`⚠️ Sender socket ${senderSocketId} not found.`);
      return res.status(400).json({ error: "Sender socket not found." });
    }
    senderSocket.leave(chatRoom);
    console.log(`Socket ${sender} left room ${chatRoom}`);
    res.status(200).json({ message: `User ${sender} left ${chatRoom}` });
  } catch (error) {
    console.error("Error sending message:", error);
    res.status(500).json({ error: "Server error" });
  }
};


exports.sendMessageSocket = async (userId,receiver,service,offer,content,io) => {
  try {
    const sender = userId;

    const io = req.app.get("socketio");
    const chatNamespace = io.of("/SocketChat");

    const conversation = await check_conversation(
      sender,
      receiver,
      service,
      offer
    );
    const conversation_id = conversation._id;
    const chatRoom = `chat_${conversation_id}`;

    const userSockets = req.app.get("userSockets");
    const senderSocketId = userSockets.get(sender);

    console.log(
      "All Connected Sockets:",
      Array.from(chatNamespace.sockets.keys())
    );

    if (!senderSocketId) {
      console.log(`⚠️ Sender ${sender} is not connected to any socket.`);
      return res.status(400).json({ error: "Sender is not connected." });
    }

    const senderSocket = chatNamespace.sockets.get(senderSocketId);

    if (typeof content !== "undefined") {
      if (!conversation || conversation.status === false) {
        return res.status(400).json({ error: "Conversation is closed." });
      }
      await send_message(sender, conversation_id, content);
    }
    /// console.log('senderSocket',senderSocket);
    console.log("senderSocketId", senderSocketId);
    senderSocket.join(chatRoom);
    console.log(`🔵 Sender ${sender} joined ${chatRoom}`);
    const room = io.sockets.adapter.rooms.get(chatRoom);

    if (room) {
      if (room.size == 2) {
        const receiverSocketId = userSockets.get(receiver);
        if (receiverSocketId) {
          const receiverSocket = io.sockets.sockets.get(receiverSocketId);
          if (receiverSocket) {
            const chatDataForReceiver = await GetMessage(
              conversation_id,
              receiver
            );
            receiverSocket.emit(
              "Message",
              JSON.stringify({ Message: chatDataForReceiver })
            );
          }
        }
      } else if (room.size == 1 && content) {
        await send_notification(sender, receiver, content, service, "message");
      }
    }
    const chatData = await GetMessage(conversation_id, sender);
    senderSocket.emit("Message", JSON.stringify({ Message: chatData }));

    // const home = await getHomeConversations(receiver);
    // io.to(`chat_user_${receiver}`).emit("Home", JSON.stringify({ home }));

    res.status(201).json({ message: "Message sent" });
  } catch (error) {
    console.error("Error sending message:", error);
    res.status(500).json({ error: "Server error" });
  }
};