const { notification } = require("../utils/logic/helper");
const UserModel = require("../models/UserModel");
const OfferModel = require("../models/OfferModel");
const ServiceModel = require("../models/ServiceModel");
const NotifiactionModel = require("../models/NotifiactionModel");
const NotificationResource = require("../utils/resources/NotificationResource");

exports.send_notification = async (
  sender,
  receiver,
  content,
  service,
  type,
  offerId
) => {
  let data;
  console.log("type", type);
  if (type === "message") {
    const SenderModel = await UserModel.findById(sender);
    const ReceiverModel = await UserModel.findById(receiver);
    console.log("offffffffferif", offerId);
    let request_to_chat =
      ReceiverModel.language === "ar" ? "رسالة جديدة من " : "New message from ";

    data = {
      title: request_to_chat + SenderModel.name,
      body: `${String(content)}`,
      related_id: String(sender),
      type: "message",
      service,
      receiver,
      offer: offerId,
    };
    await notification(ReceiverModel.fcm_token, data);
  } else if (type === "new_service") {
    console.log("new_service");
    const serviceModel = await ServiceModel.findById(service).populate("user");
    if (!serviceModel) return; // Ensure service exists

    const users = await UserModel.find({
      selectedCategories: serviceModel.category_id,
    });

    for (let user of users) {
      const request_to_title =
        user.language === "ar" ? "خدمة جديدة من " : "New pending service from ";
      console.log("user name :", user.name);

      data = {
        title: request_to_title + serviceModel.user.name,
        body: `${String(serviceModel.description)}`,
        related_id: String(service),
        type: "new_service",
        service,
        receiver: user._id,
      };

      await notification(user.fcm_token, data);
    }
  } else if (type === "new_offer") {
    console.log("new_offer");
    const serviceModel = await ServiceModel.findById(service).populate("user");
    const user = await UserModel.findById(sender);
    if (!serviceModel || !serviceModel.user || !serviceModel.user.fcm_token)
      return; // Ensure service exists

    const request_to_title =
      serviceModel.user.language === "ar" ? "عرض جديد من " : "New offer from ";

    data = {
      title: request_to_title + user.name,
      body: `${String(serviceModel.description)}`,
      related_id: String(service),
      type: "new_offer",
      service,
      receiver: serviceModel.user._id,
    };
    console.log("[new_offer] Sending to:", serviceModel.user.name);
    await notification(serviceModel.user.fcm_token, data);
  } else if (type === "accepted") {
    const request_to_title =
      receiver.language === "ar"
        ? `قام ${sender.name} بالموافقة على عرضك`
        : `${sender.name} has approved your offer`;

    data = {
      title: request_to_title,
      body: `${String(service.description)}`,
      related_id: String(content._id),
      type: "accepted",
      service: String(service._id),
      receiver: String(receiver._id),
    };
    console.log("[accepted] Sending to:", receiver.name);
    await notification(receiver.fcm_token, data);
  } else if (type === "rejected") {
    const request_to_title =
      receiver.language === "ar"
        ? `قام ${sender.name} برفض عرضك`
        : `${sender.name} has rejected your offer`;

    data = {
      title: request_to_title,
      body: `${String(service.description)}`,
      related_id: String(content._id),
      type: "rejected",
      service: String(service._id),
      receiver: String(receiver._id),
    };
    console.log("[accepted] Sending to:", receiver.name);
    await notification(receiver.fcm_token, data);
  } else if (type === "review") {
    const request_to_title =
      receiver.language === "ar"
        ? `قام ${sender.name} بإضافة تقييم لك`
        : `${sender.name} has added a review for you`;

    data = {
      title: request_to_title,
      body: `${String(content)}`,
      related_id: String(service._id),
      type: "review",
      service: String(service._id),
      receiver: String(receiver._id),
    };
    console.log("[review] Sending to:", receiver.name);
    await notification(receiver.fcm_token, data);
  } else if (type === "new_request_service") {
    console.log("new_request_service");

    const user = await UserModel.findById(receiver);
    const serviceModel = await ServiceModel.findById(service).populate("user");
    if (!serviceModel) return;

    const serviceUserName = serviceModel.user.name;
    console.log("serviceUserName");
    const request_to_title =
      user.language === "ar"
        ? `لديك طلب خدمة من ${serviceUserName} جديد الرجاء المراجعة`
        : `You have a new service request from ${serviceUserName}. Please review it`;

    console.log("request_to_title:", request_to_title);

    data = {
      title: request_to_title,
      body: `${String(serviceModel.description)}`,
      related_id: String(sender),
      type: "message",
      service,
      receiver,
      offer: offerId,
    };

    await notification(user.fcm_token, data);
  }
};

exports.GetMyNotification = async (req, res, next) => {
  const userId = req.user._id;
  const Notification = await NotifiactionModel.find({
    user: userId,
    status: 1,
  }).sort({
    createdAt: -1,
  });

  const data = await NotificationResource.collection(Notification);
  res.status(200).json({ data: data });
};

exports.deleteNotification = async (req, res) => {
  const { ids } = req.body;

  await NotifiactionModel.updateMany(
    {
      _id: { $in: ids },
      user: req.user._id,
    },
    { $set: { status: 0 } }
  );

  res.status(200).json({ message: "Notifications deleted successfully." });
};
