const OfferModel = require("../models/OfferModel");
const ServiceModel = require("../models/ServiceModel");
const { NewServiceSocket } = require("./ServiceController");
const ApiError = require("../utils/apiError");
const OfferResource = require("../utils/resources/OfferResource");
const MyOfferResource = require("../utils/resources/auth/OfferResource");
const { send_notification } = require("./NotificationController");
const { GetServicesOfUserocket, GetDetails } = require("./ServiceController");

exports.GetOffer = async (req, res, next) => {
  const { id } = req.params;

  const offer = await OfferModel.find({
    service: id,
    OfferStatus: { $in: ["pending", "accepted"] },
  }).populate("user");

  if (!offer) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }

  const data = await OfferResource.collection(offer);
  res.status(200).json({ data: data });
};

exports.StatusOffer = async (req, res) => {
  const { status } = req.body;
  const { offer } = req;
  const io = req.app.get("socketio");
  const serviceNamespace = io.of("/SocketService");

  if (status === "accepted") {
    offer.service.ServiceStatus = "accepted";
    await offer.service.save();
    await OfferModel.updateMany(
      {
        service: offer.service,
        _id: { $ne: offer._id },
        OfferStatus: "pending",
      },
      { $set: { OfferStatus: "not_accepted" } }
    );
    console.log("offerrrrrrr", offer);
    const allOffers = await OfferModel.find({
      service: offer.service,
    });
    offer.OfferStatus = status;
    await offer.save();
    for (const offer of allOffers) {
      const newList = await this.GetMyOfferSocket(offer.user._id);
      serviceNamespace.to(`user_${offer.user._id}`).emit("Offer", newList);
    }
    await send_notification(
      offer.service.user,
      offer.user,
      offer,
      offer.service,
      "accepted"
    );
  } else if (status == "rejected") {
    offer.OfferStatus = status;
    await offer.save();

    const newList = await this.GetMyOfferSocket(offer.user._id);
    serviceNamespace.to(`user_${offer.user._id}`).emit("Offer", newList);

    const services = await NewServiceSocket(
      offer.user.selectedCategories,
      offer.user._id
    );
    serviceNamespace.to(`user_${offer.user._id}`).emit("services", services);
    await send_notification(
      offer.service.user,
      offer.user,
      offer,
      offer.service,
      "rejected"
    );
  } else {
    offer.OfferStatus = status;
    await offer.save();
    const newList = await this.GetMyOfferSocket(offer.user._id);
    serviceNamespace.to(`user_${offer.user._id}`).emit("Offer", newList);
    const services = await NewServiceSocket(
      offer.user.selectedCategories,
      offer.user._id
    );
    serviceNamespace.to(`user_${offer.user._id}`).emit("services", services);
  }

  return res.status(200).json({
    message: `Offer status updated to ${status}`,
  });
};

exports.GetOfferById = async (req, res, next) => {
  const { id } = req.params;

  const offer = await OfferModel.findById(id).populate({
    path: "service",
    populate: {
      path: "user",
    },
  });

  if (!offer) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }

  const data = await new MyOfferResource(offer).toJSON();
  res.status(200).json({ data });
};

exports.GetMyOfferSocket = async (userId, status) => {
  const offer = await OfferModel.find({
    user: userId,
    ...(status && {
      OfferStatus: { $in: Array.isArray(status) ? status : [status] },
    }),
  })
    .populate({
      path: "service",
      populate: { path: "user" },
    })
    .sort({
      createdAt: -1,
    });
  if (!offer) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }

  const data = await MyOfferResource.collection(offer);
  return data;
};

exports.UpdateOfferSocket = async (offerId, price) => {
  try {
    await OfferModel.findOneAndUpdate(
      { _id: offerId },
      { price },
      { new: true, runValidators: true }
    );

    return "Offer update successfully";
  } catch (error) {
    console.error("Error adding offer:", error);
    res.status(500).json({ error: error.message });
  }
};

exports.UpdateStatusOfferSocket = async (offerId, status) => {
  try {
    const offer = await OfferModel.findById(offerId);
    if (!offer) throw new Error("Offer not found.");

    const service = await ServiceModel.findById(offer.service);
    if (!service) throw new Error("Related service not found.");

    // Update statuses
    offer.OfferStatus = status;
    await offer.save();

    service.ServiceStatus = status;
    await service.save();

    return "Offer update successfully";
  } catch (error) {
    console.error("Error adding offer:", error);
  }
};

exports.AddOfferSocket = async (userId, description, price, serviceId) => {
  try {
    await OfferModel.findOneAndUpdate(
      { user: userId, service: serviceId },
      { description, price, OfferStatus: "pending" },
      {
        new: true, // Return the updated doc
        upsert: true, // Create if not exists
        runValidators: true, // Validate schema
        setDefaultsOnInsert: true, // Set default values if inserting
      }
    );
    await send_notification(userId, null, null, serviceId, "new_offer");
    return "Offer created successfully";
  } catch (error) {
    console.error("Error adding offer:", error);
  }
};

exports.StatusOfferSocket = async (socket, offerId, status, DropReasons) => {
  const offer = await OfferModel.findById(offerId)
    .populate("service")
    .populate("user");

  if (status === "drop") {
    offer.service.ServiceStatus = "active";
    offer.service.save();
    await OfferModel.updateMany(
      {
        service: offer.service._id,
        OfferStatus: "not_accepted",
      },
      { $set: { OfferStatus: "pending" } }
    );
    offer.drop_reasons = DropReasons;
    offer.OfferStatus = status;
    offer.save();
    const allOffers = await OfferModel.find({
      service: offer.service._id,
    });
    for (const offer of allOffers) {
      const newList = await this.GetMyOfferSocket(offer.user._id);
      socket.to(`user_${offer.user._id}`).emit("Offer", newList);
    }
    const ServiceOwner = await GetServicesOfUserocket(offer.service.user);
    socket.to(`user_${offer.service.user}`).emit("MyServices", ServiceOwner);

    const Details = await GetDetails(offer.service._id);
    socket.to(`service_${offer.service._id}`).emit("Details", Details);

    return "drop sucess";
  } else if (status === "canceled") {
    await OfferModel.findByIdAndUpdate(offerId, {
      $set: { OfferStatus: "canceled" },
    });
    const ServiceOwner = await GetServicesOfUserocket(offer.service.user);
    socket.to(`user_${offer.service.user}`).emit("MyServices", ServiceOwner);

    const Details = await GetDetails(offer.service._id);
    socket.to(`service_${offer.service._id}`).emit("Details", Details);
  } else if (status === "accepted") {
    offer.service.ServiceStatus = "accepted";
    await offer.service.save();

    await OfferModel.updateMany(
      {
        service: offer.service,
        _id: { $ne: offer._id },
        OfferStatus: "pending",
      },
      { $set: { OfferStatus: "not_accepted" } }
    );
    offer.OfferStatus = status;
    await offer.save();

    const allOffers = await OfferModel.find({
      service: offer.service,
    });
    console.log("offf42114214243fer", allOffers);
    for (const offer of allOffers) {
      const newList = await this.GetMyOfferSocket(offer.user._id);
      socket.to(`user_${offer.user._id}`).emit("Offer", newList);
    }
    await send_notification(
      offer.service.user,
      offer.user,
      offer,
      offer.service,
      "accepted"
    );

    const ServiceOwner = await GetServicesOfUserocket(offer.service.user);
    socket.to(`user_${offer.service.user}`).emit("MyServices", ServiceOwner);

    const Details = await GetDetails(offer.service._id);
    socket.to(`service_${offer.service._id}`).emit("Details", Details);
  } else if (status === "rejected") {
    offer.OfferStatus = status;
    await offer.save();
    const newList = await this.GetMyOfferSocket(offer.user._id);
    socket.to(`user_${offer.user._id}`).emit("Offer", newList);

    const services = await NewServiceSocket(
      offer.user.selectedCategories,
      offer.user._id
    );
    socket.to(`user_${offer.user._id}`).emit("services", services);
    await send_notification(
      offer.service.user,
      offer.user,
      offer,
      offer.service,
      "rejected"
    );
    const ServiceOwner = await GetServicesOfUserocket(offer.service.user);
    socket.to(`user_${offer.service.user}`).emit("MyServices", ServiceOwner);

    const Details = await GetDetails(offer.service._id);
    socket.to(`service_${offer.service._id}`).emit("Details", Details);
  }
};
