const RatingModel = require("../models/RatingModel");
const UserModel = require("../models/UserModel");
const ServiceModel = require("../models/ServiceModel");
const OfferModel = require("../models/OfferModel");
const UserResource = require("../utils/resources/UserResource");
const ReviewResource = require("../utils/resources/ReviewResource");
const { send_notification } = require("./NotificationController");
const { GetPostOfUser } = require("../utils/logic/helper");
exports.Add_rating = async (req, res) => {
  const { serviceId, offerId, ratedUser, rating, review } = req.body;
  const status = req.status;
  const ratedBy = req.user._id;

  let validStatuses = ["accepted"];

  if (status === "Provider") {
    validStatuses.push("ended");
  }

  const service = await ServiceModel.findOne({
    _id: serviceId,
    ServiceStatus: { $in: validStatuses },
  }).populate("user");

  if (!service) {
    throw new Error(
      `Service not in a valid status (${validStatuses.join(", ")})`
    );
  }

  let validStatusesOffer = ["accepted"];

  if (req.status === "Implemented") {
    validStatusesOffer.push("ended");
  }
  const offer = await OfferModel.findOne({
    _id: offerId,
    OfferStatus: { $in: validStatusesOffer },
  }).populate("user");

  if (!offer) {
    throw new Error(
      `Offer not in a valid status (${validStatusesOffer.join(", ")})`
    );
  }

  try {
    const added = await RatingModel.create({
      service: serviceId,
      offer: offerId,
      ratedBy,
      ratedUser,
      rating,
      review,
      status: req.status,
    });

    if (status === "Provider") {
      req.offer.OfferStatus = "ended";
      await req.offer.save();
      await send_notification(
        offer.user,
        service.user,
        review,
        added,
        "review"
      );
    } else if (status === "Implemented") {
      req.service.ServiceStatus = "ended";
      await req.service.save();
      await send_notification(
        service.user,
        offer.user,
        review,
        added,
        "review"
      );
    }
    res.status(200).json({ message: "Rating submitted successfully!" });
  } catch (error) {
    console.log(error.message);
    res.status(500).json({ message: error.message });
  }
};

exports.reviews = async (req, res) => {
  const userId = req.params.id;
  const status = req.params.status;
  try {
    const reviews = await RatingModel.find({
      ratedUser: userId,
      status,
    })
      .populate("ratedBy", "name")
      .sort({
        createdAt: -1,
      });

    const offers_count = await OfferModel.countDocuments({
      user: userId,
    });
    const service_count = await ServiceModel.countDocuments({
      user: userId,
    });
    const lang = req.headers["accept-language"] || "en";
    const user = await UserModel.findById(userId).populate(
      "selectedCategories",
      "name_ar name_en"
    );
    const userResource = await new UserResource(user, status, lang).toJSON();

    const totalReviews = reviews.length;
    const avgRating =
      totalReviews > 0
        ? reviews.reduce((sum, review) => sum + review.rating, 0) / totalReviews
        : 0;
    const reviewResource = await ReviewResource.collection(reviews);
    const post = await GetPostOfUser(req.params.id);
    res.status(200).json({
      totalReviews,
      avgRating: avgRating.toFixed(2),
      offers_count,
      service_count,
      userResource,
      post,
      reviewResource,
    });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

exports.GetReviewById = async (req, res) => {
  const { id } = req.params;
  const review = await RatingModel.findById(id)
    .populate("service", "description image price ServiceStatus")
    .populate("offer", "description price OfferStatus")
    .populate("ratedUser", "name image");
  if (!review) {
    throw new Error(`not found ${req.originalUrl}`);
  }
  res.status(200).json(review);
};
