const mongoose = require("mongoose");

const LocationSchema = new mongoose.Schema({
  long: {
    type: String,
    required: true,
  },
  lang: {
    type: String,
    required: true,
  },
  name: {
    type: String,
    required: true,
  },
});

const ServiceSchema = new mongoose.Schema(
  {
    category_id: {
      type: mongoose.Schema.Types.ObjectId,
      ref: "Category",
      required: [true, "Category id is required"],
    },
    description: {
      type: String,
      required: [true, "Description is required"],
    },
    price: {
      type: Number,
      required: [true, "Price is required"],
    },
    images: {
      type: [String],
    },
    locations: {
      type: [LocationSchema],
      default: [],
    },
    ServiceStatus: {
      type: String,
      enum: ["active", "canceled", "accepted", "ended"],
      default: "active",
    },
    user: {
      type: mongoose.Schema.Types.ObjectId,
      ref: "User",
      require: true,
    },
    private: {
      type: mongoose.Schema.Types.ObjectId,
      ref: "User",
    },
  },
  { timestamps: true }
);

const ServiceModel = mongoose.model("Service", ServiceSchema);

module.exports = ServiceModel;
