const mongoose = require("mongoose");

const UserSchema = new mongoose.Schema(
  {
    name: {
      type: String,
      required: [true, "Name is required"],
      minlength: [3, "Too short name"],
      maxlength: [11, "Too long name"],
    },
    phone: {
      type: String,
      required: [true, "Phone is required"],
      unique: true,
      match: [/^\+?\d{10,15}$/, "Invalid phone number format"],
    },
    isPhoneVerified: {
      type: Boolean,
      default: false,
    },
    image: {
      type: String,
    },
    status: {
      type: Boolean,
      default: false,
    },
    role: {
      type: String,
      enum: ["admin", "user", "google_play"],
      default: "user",
    },
    language: {
      type: String,
      enum: ["ar", "en"],
      default: "en",
    },
    fcm_token: {
      type: String,
    },
    sessionVersion: {
      type: Number,
      default: 1,
    },
    selectedCategories: [
      { type: mongoose.Schema.Types.ObjectId, ref: "Category" },
    ],
  },
  { timestamps: true }
);

const CategoryModel = mongoose.model("User", UserSchema);

module.exports = CategoryModel;
