const UserModel = require("../../models/UserModel");

class CategoryResource {
  constructor(category, lang, user_id) {
    this.category = category;
    this.lang = lang;
    this.user_id = user_id;
  }

  async toJSON() {
    try {
      const user = await UserModel.findById(this.user_id).select("selectedCategories");

      return {
        id: this.category._id,
        name: this.category[`name_${this.lang}`] || this.category["name_en"], // Default fallback
        slug: this.category.slug,
        status: this.category.status,
        numberOfLocations: this.category.numberOfLocations,
        isFavorite: user ? user.selectedCategories.includes(this.category._id) : false, // Ensure user exists
        image: this.category.image || null,
        createdAt: this.category.createdAt,
      };
    } catch (error) {
      console.error("Error fetching user:", error);
      return null; // Return null in case of an error
    }
  }

  static async collection(categories, lang, user_id) {
    const promises = categories.map(async category => {
      const resource = new CategoryResource(category, lang, user_id);
      return await resource.toJSON();
    });

    return Promise.all(promises); // Resolve all promises before returning
  }
}

module.exports = CategoryResource;
