const CategoryResource = require("./CategoryResource");
const UserResource = require("./UserResource");
class PostResource {
  constructor(post) {
    this.post = post;
  }

  async toJSON() {
    try {
      if (!this.post) {
        console.error("post is undefined or null:", this.post);
        return null;
      }

      const User = this.post.user
      ? await new UserResource(this.post.user, "Implemented").toJSON()
      : null;
      
      const Category = this.post.category
        ? await new CategoryResource(
            this.post.category,
            this.post.user.language ?? "en",
            this.post.user._id
          ).toJSON()
        : null;

      return {
        id: this.post._id,
        description: this.post.description,
        Category,
        User,
        order_count:this.post.order_count,
        price: this.post.price,
        images: this.post.images || null,
        createdAt: this.post.createdAt,
      };
    } catch (error) {
      console.error("Error fetching post:", error);
      return null;
    }
  }

  static async collection(posts) {
    const promises = posts.map(async (post) => new PostResource(post).toJSON());
    return Promise.all(promises);
  }
}

module.exports = PostResource;
